package org.todo.model.user;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlElementWrapper;
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlProperty;
import com.fasterxml.jackson.dataformat.xml.annotation.JacksonXmlRootElement;
import org.todo.model.todo.Todo;
import org.todo.util.ObjectMapperFactory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

@JacksonXmlRootElement(localName = "users")
public class UserAdmin {

	private static final File DATA_FILE = new File(System.getProperty("user.home"), "todo-data.xml");

	private static final ObjectMapper objectMapper = ObjectMapperFactory.createXmlMapper();
	private static final Logger logger = Logger.getLogger(UserAdmin.class.getName());
	private static UserAdmin instance;

	@JacksonXmlElementWrapper(useWrapping = false)
	@JacksonXmlProperty(localName = "user")
	private final List<User> users = new ArrayList<>();

	public static UserAdmin getInstance() {
		if (instance == null) {
			try (Reader reader = new FileReader(DATA_FILE)) {
				instance = objectMapper.readValue(reader, UserAdmin.class);
			} catch (FileNotFoundException ex) {
				instance = new UserAdmin();
			} catch (IOException ex) {
				throw new RuntimeException(ex);
			}
		}
		return instance;
	}

	private UserAdmin() {
		User user = new User("test", "12345");
		Todo todo = new Todo("Todo", "Test", LocalDate.now());
		user.getTodoList().addTodo(todo);
		users.add(user);
	}

	public synchronized User registerUser(String username, String password) throws UserAlreadyExistsException {
		if (findUser(username) != null) {
			throw new UserAlreadyExistsException();
		}
		User user = new User(username, password);
		users.add(user);
		save();
		logger.info("User " + username + " registered");
		return user;
	}

	public User loginUser(String username, String password) throws InvalidCredentialsException {
		User user = findUser(username);
		if (user == null || !user.getPassword().equals(password)) {
			throw new InvalidCredentialsException();
		}
		logger.info("User " + username + " logged in");
		return user;
	}

	private User findUser(String username) {
		return users.stream().filter(user -> user.getName().equals(username)).findFirst().orElse(null);
	}

	public void save() {
		try (Writer writer = new FileWriter(DATA_FILE)) {
			objectMapper.writeValue(writer, this);
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
	}
}
